@@ -6,9 +6,11 @@ from tabulate import tabulate
 import string
 from fractions import Fraction
 from mpmath import *
+from tkinter import messagebox
 import numpy as np
 import math
 import cmath
+import random
 import time
 import sys
 import keyboard
@@ -16,6 +18,7 @@ import matplotlib.pyplot as plt
 from mpl_toolkits import mplot3d
 import inspect
 import re
+import mpmath
 
 operatorsDict = {"*": 2, "/": 2, "+": 1, "%": 2, '': 4, "-": 1}
 operators = operatorsDict.keys()
@@ -379,7 +382,7 @@ def listMultiply(List):
         y+=1
     return x
 def power(a, *args):    
-    b = mult(args)
+    b = multiply(args)
     x = a**(b)
     if b < 1:
         return root(a, 1/b)
@@ -510,6 +513,12 @@ def perfectSquare(num):
     if math.trunc(root)-root==0:
         return True
     return False
+
+def perfectSquareCheck(num):
+    if perfectSquare(num) == True:
+        return True
+    return False
+
 def perfectSquarePrinter(low, high):
     for i in range(low, high):
         if perfectSquare(i) == True:
@@ -602,13 +611,13 @@ def product(n, a, b):
     return listMultiply(nthRange(n, a, b))
 
 def PercentagetoDecimal(percentage):
-    return p / 100
+    return percentage / 100
 def PerToFrac(p):
     dec = p / 100
     fraction = Fraction(dec)
     return str(fraction)
 def PercentagetoFraction(percentage):
-    fracOutput = PertoFrac(p)
+    fracOutput = PerToFrac(percentage)
     return "{0}".format(fracOutput)
 def FractiontoDecimal(fraction):
     dec = f
@@ -616,10 +625,10 @@ def FractiontoDecimal(fraction):
 def FractiontoPercentage(fraction):
     return f * 100
 def DecimaltoPercentage(decimal):
-    percentage = "{:.0%}".format(d)
+    percentage = "{:.0%}".format(decimal)
     return str(percentage)
 def DecimaltoFraction(decimal):
-    return str(Fraction(d).limit_denominator())
+    return str(Fraction(decimal).limit_denominator())
 
 def triangleArea(l, w):
     return (l*w)/2
@@ -717,7 +726,7 @@ def lgamma(a):
 
 def pytheoromside(a, b):
     if a > c:
-        a1 = a*ad
+        a1 = a*a
         b1 = b*b
         c1 = a1 - b1
         c = math.sqrt(c1)
@@ -748,8 +757,8 @@ def pyTheoremCB(c, b):
 def cos_rule(c, b, A):
     c1 *= c
     b1 *= b
-    A1 = math.cos(A)
-    thing = A1*c*b
+    a1 = math.cos(A)
+    thing = a1*c*b
     ans = b1 + a1 - thing
     answer = math.sqrt(answer)
     return answer
@@ -1056,8 +1065,8 @@ def graph(*args, lrangex=-10, urangex=10, lrangey=-10, urangey=10, graph_points=
     i = 0
     x = np.linspace(lrangex, urangex, graph_points)
     while i < len(args):
-        if type(eval(args[i])) == np.ndarray:
-            y = eval(args[i])
+        if type(evaluateExpression(args[i])) == np.ndarray:
+            y = evaluateExpression(args[i])
         else:
             pass
         fig = plt.figure()
@@ -1091,37 +1100,9 @@ def isInt(x, gaussian = False):
 def Zeta(s):
     return zeta(s)
 
-def graph(*args, lrangex=-10, urangex=10, lrangey=-10, urangey=10, graph_points=100, gridset=True, scaling=True, graph_title=None, xtitle=None, ytitle=None):
-    i = 0
-    x = np.linspace(lrangex, urangex, graph_points)
-    while i < len(args):
-        if type(eval(args[i])) == np.ndarray:
-            y = eval(args[i])
-        else:
-            pass
-        fig = plt.figure()
-        ax = fig.add_subplot(1, 1, 1)
-        ax.spines["left"].set_position("center")
-        ax.spines["bottom"].set_position("zero")
-        ax.spines["right"].set_color("none")
-        ax.spines["top"].set_color("none")
-        ax.xaxis.set_ticks_position("bottom")
-        ax.yaxis.set_ticks_position("left")
-        plt.plot(x,y, "-b", label=args)
-        i+=1
-    plt.title(graph_title)
-    plt.xlabel(xtitle,fontsize=20)
-    plt.ylabel(ytitle,fontsize=20)
-    plt.autoscale(enable=scaling)
-    plt.grid(gridset)
-    if lrangex or urangex != False:
-        plt.xlim([lrangex,urangex])
-    if lrangey or urangey != False:
-        plt.ylim([lrangey,urangey])
-    plt.show()
 def f(fx, x, y):
-    if type(eval(fx)) == np.ndarray:
-        return eval(fx)
+    if type(evaluateExpression(fx)) == np.ndarray:
+        return evaluateExpression(fx)
     else:
         pass
     
@@ -1147,7 +1128,8 @@ def graph3dcontour(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, lran
     if lrangey != False or urangey == False:
         ax.set_zlim(lrangey, urangey)
     return plt.show()
-def linein3d(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, color='gray'):
+
+def linein3d(*args, lrangex=-10, lrangey=-10, lrangez = -10, urangex=10, urangey=10, urangez = 10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, color='gray'):
     x = np.linspace(lrangex, urangex, graph_points)
     y = np.linspace(lrangey, urangey, graph_points)
     fig = plt.figure()
@@ -1157,12 +1139,12 @@ def linein3d(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, graph_poin
         lis = args[i]
         fz = lis[0]
         fy = lis[1]
-        if type(eval(fz)) == np.ndarray:
-            zline = eval(fz)
+        if type(evaluateExpression(fz)) == np.ndarray:
+            zline = evaluateExpression(fz)
         else:
             pass
-        if type(eval(fy)) == np.ndarray:
-            yline = eval(fy)
+        if type(evaluateExpression(fy)) == np.ndarray:
+            yline = evaluateExpression(fy)
         else:
             pass
         ax.plot3D(x, yline, zline, color)
@@ -1178,6 +1160,7 @@ def linein3d(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, graph_poin
     if lrangey != False or urangey == False:
         ax.set_zlim(lrangey, urangey)
     return plt.show()
+
 def graph3dwire(*args, lrangex=-10, lrangey=-10, urangex=10, urangey=10, lrangez=-10, urangez=10, graph_points=1000, lablex='x', labley='y', lablez='z', graph_title=None, edgecolor=False, color='black', fill='viridis'):
     x = np.linspace(lrangex, urangex, graph_points)
     y = np.linspace(lrangey, urangey, graph_points)
